#ifndef PRIVATE_RENDERING_H
#define PRIVATE_RENDERING_H

#include "../rendering.h"
#include "../textures.h"
#include "../util.h"

#include <SDL2/SDL.h>
#include <glad/glad.h>

#include <stdbool.h>

struct sprite_primitive {
    t_frect rect;
    t_color color;
    double rotation;
    SDL_BlendMode blend_mode;
    t_texture_key texture_key;
    int layer;
    bool flip_x;
    bool flip_y;
};

struct rect_primitive {
    t_frect rect;
    t_color color;
};

struct circle_primitive {
    float radius;
    t_color color;
    t_fvec2 position;
};

/* union for in-place recalculation of texture coordinates */
union uncolored_space_triangle {
    /* pending for sending, uvs are not final as texture atlases could update */
    struct uncolored_space_triangle_primitive {
        t_fvec3 v0;
        t_fvec2 uv0; /* in pixels */
        t_fvec3 v1;
        t_fvec2 uv1; /* in pixels */
        t_fvec3 v2;
        t_fvec2 uv2; /* in pixels */
    } primitive;

    /* structure that is passed in opengl vertex array */
    struct uncolored_space_triangle_payload {
        t_fvec3 v0;
        t_fvec2 uv0;
        t_fvec3 v1;
        t_fvec2 uv1;
        t_fvec3 v2;
        t_fvec2 uv2;
    } payload;
};

/* batch of primitives with overlapping properties */
struct mesh_batch {
    GLuint buffer;          /* server side storage */
    size_t buffer_len;      /* element count */
    uint8_t *primitives;
};

struct mesh_batch_item {
    t_texture_key key;
    struct mesh_batch value;
};

#endif
